@@ -28,6 +28,7 @@ import six
 from stevedore import driver
 
 import tooz
+from tooz import _retry
 
 LOG = logging.getLogger(__name__)
 
@@ -150,6 +151,8 @@ class Heart(object):
                  event_cls=threading.Event):
         self._thread_cls = thread_cls
         self._dead = event_cls()
+        self._finished = event_cls()
+        self._finished.set()
         self._runner = None
         self._driver = driver
         self._beats = 0
@@ -162,33 +165,38 @@ class Heart(object):
     def is_alive(self):
         """Returns if the heart is beating."""
         return not (self._runner is None
-                    or not self._runner.is_alive())
+                    or not self._runner.is_alive()
+                    or self._finished.is_set())
 
     @excutils.forever_retry_uncaught_exceptions
     def _beat_forever_until_stopped(self):
         """Inner beating loop."""
-        while not self._dead.is_set():
-            with timeutils.StopWatch() as w:
-                wait_until_next_beat = self._driver.heartbeat()
-            ran_for = w.elapsed()
-            if ran_for > wait_until_next_beat:
-                LOG.warning(
-                    "Heartbeating took too long to execute (it ran for"
-                    " %0.2f seconds which is %0.2f seconds longer than"
-                    " the next heartbeat idle time). This may cause"
-                    " timeouts (in locks, leadership, ...) to"
-                    " happen (which will not end well).", ran_for,
-                    ran_for - wait_until_next_beat)
-            self._beats += 1
-            # NOTE(harlowja): use the event object for waiting and
-            # not a sleep function since doing that will allow this code
-            # to terminate early if stopped via the stop() method vs
-            # having to wait until the sleep function returns.
-            self._dead.wait(wait_until_next_beat)
+        try:
+            while not self._dead.is_set():
+                with timeutils.StopWatch() as w:
+                    wait_until_next_beat = self._driver.heartbeat()
+                ran_for = w.elapsed()
+                if ran_for > wait_until_next_beat:
+                    LOG.warning(
+                        "Heartbeating took too long to execute (it ran for"
+                        " %0.2f seconds which is %0.2f seconds longer than"
+                        " the next heartbeat idle time). This may cause"
+                        " timeouts (in locks, leadership, ...) to"
+                        " happen (which will not end well).", ran_for,
+                        ran_for - wait_until_next_beat)
+                self._beats += 1
+                # NOTE(harlowja): use the event object for waiting and
+                # not a sleep function since doing that will allow this code
+                # to terminate early if stopped via the stop() method vs
+                # having to wait until the sleep function returns.
+                self._dead.wait(wait_until_next_beat)
+        finally:
+            self._finished.set()
 
     def start(self, thread_cls=None):
         """Starts the heart beating thread (noop if already started)."""
         if not self.is_alive():
+            self._finished.clear()
             self._dead.clear()
             self._beats = 0
             if thread_cls is None:
@@ -203,8 +211,8 @@ class Heart(object):
 
     def wait(self, timeout=None):
         """Wait up to given timeout for the heart beating thread to stop."""
-        self._runner.join(timeout)
-        return self._runner.is_alive()
+        self._finished.wait(timeout)
+        return self._finished.is_set()
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -377,6 +385,8 @@ class CoordinationDriver(object):
         if self.requires_beating and start_heart:
             self.heart.start()
         self._started = True
+        # Tracks which group are joined
+        self._joined_groups = set()
 
     def _start(self):
         pass
@@ -392,6 +402,17 @@ class CoordinationDriver(object):
         if self.heart.is_alive():
             self.heart.stop()
             self.heart.wait()
+        leaving = [self.leave_group(group)
+                   for group in self._joined_groups]
+        for leave in leaving:
+            try:
+                leave.get()
+            except ToozError:
+                # Whatever happens, ignore. Maybe we got booted out/never
+                # existed in the first place, or something is down, but we just
+                # want to call _stop after whatever happens to not leak any
+                # connection.
+                pass
         self._stop()
         self._started = False
 
@@ -431,6 +452,33 @@ class CoordinationDriver(object):
         """
         raise tooz.NotImplemented
 
+    @_retry.retry()
+    def join_group_create(self, group_id):
+        """Join a group and create it if necessary.
+
+        If the group cannot be joined because it does not exist, it is created
+        before being joined.
+
+        This function will keep retrying until it can create the group and join
+        it. Since nothing is transactional, it may have to retry several times
+        if another member is creating/deleting the group at the same time.
+
+        :param group_id: Identifier of the group to join and create
+
+        """
+        req = self.join_group(group_id)
+        try:
+            req.get()
+        except GroupNotCreated:
+            req = self.create_group(group_id)
+            try:
+                req.get()
+            except GroupAlreadyExist:
+                # The group might have been created in the meantime, ignore
+                pass
+            # Now retry to join the group
+            raise _retry.TryAgain
+
     @staticmethod
     def leave_group(group_id):
         """Leave a group asynchronously.
@@ -455,9 +503,9 @@ class CoordinationDriver(object):
 
     @staticmethod
     def get_members(group_id):
-        """Return the list of all members ids of the specified group.
+        """Return the set of all members ids of the specified group.
 
-        :returns: list of all created group ids
+        :returns: set of all created group ids
         :rtype: CoordAsyncResult
         """
         raise tooz.NotImplemented
@@ -573,8 +621,6 @@ class _RunWatchersMixin(object):
                         timeout=w.leftover(return_none=True))
                 except GroupNotCreated:
                     group_members = set()
-                else:
-                    group_members = set(group_members)
                 if (group_id in self._joined_groups and
                         self._member_id not in group_members):
                     self._joined_groups.discard(group_id)
