@@ -19,8 +19,8 @@ from oslo_utils import encodeutils
 import pymysql
 
 import tooz
+from tooz import _retry
 from tooz import coordination
-from tooz.drivers import _retry
 from tooz import locking
 from tooz import utils
 
@@ -52,7 +52,7 @@ class MySQLLock(locking.Lock):
             # So, we track locally the lock status with self.acquired
             if self.acquired is True:
                 if blocking:
-                    raise _retry.Retry
+                    raise _retry.TryAgain
                 return False
 
             try:
@@ -69,7 +69,7 @@ class MySQLLock(locking.Lock):
                     cause=e)
 
             if blocking:
-                raise _retry.Retry
+                raise _retry.TryAgain
             return False
 
         return _lock()
@@ -105,6 +105,7 @@ class MySQLDriver(coordination.CoordinationDriver):
     """
 
     CHARACTERISTICS = (
+        coordination.Characteristics.NON_TIMEOUT_BASED,
         coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
         coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
         coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS,
