@@ -18,12 +18,13 @@ import contextlib
 import hashlib
 import logging
 
+from oslo_utils import encodeutils
 import psycopg2
 import six
 
 import tooz
+from tooz import _retry
 from tooz import coordination
-from tooz.drivers import _retry
 from tooz import locking
 from tooz import utils
 
@@ -55,7 +56,8 @@ _DIAGNOSTICS_ATTRS = tuple([
 
 def _format_exception(e):
     lines = [
-        "%s: %s" % (type(e).__name__, utils.exception_message(e).strip()),
+        "%s: %s" % (type(e).__name__,
+                    encodeutils.exception_to_unicode(e).strip()),
     ]
     if hasattr(e, 'pgcode') and e.pgcode is not None:
         lines.append("Error code: %s" % e.pgcode)
@@ -110,7 +112,7 @@ class PostgresLock(locking.Lock):
             # so we track it internally if the process already has the lock.
             if self.acquired is True:
                 if blocking:
-                    raise _retry.Retry
+                    raise _retry.TryAgain
                 return False
 
             with _translating_cursor(self._conn) as cur:
@@ -128,7 +130,7 @@ class PostgresLock(locking.Lock):
                     elif blocking is False:
                         return False
                     else:
-                        raise _retry.Retry
+                        raise _retry.TryAgain
 
         return _lock()
 
@@ -144,7 +146,7 @@ class PostgresLock(locking.Lock):
 
     def __del__(self):
         if self.acquired:
-            LOG.warn("unreleased lock %s garbage collected" % self.name)
+            LOG.warning("unreleased lock %s garbage collected", self.name)
 
 
 class PostgresDriver(coordination.CoordinationDriver):
@@ -158,6 +160,17 @@ class PostgresDriver(coordination.CoordinationDriver):
     .. _PostgreSQL: http://www.postgresql.org/
     """
 
+    CHARACTERISTICS = (
+        coordination.Characteristics.NON_TIMEOUT_BASED,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS,
+    )
+    """
+    Tuple of :py:class:`~tooz.coordination.Characteristics` introspectable
+    enum member(s) that can be used to interogate how this driver works.
+    """
+
     def __init__(self, member_id, parsed_url, options):
         """Initialize the PostgreSQL driver."""
         super(PostgresDriver, self).__init__()
@@ -200,9 +213,9 @@ class PostgresDriver(coordination.CoordinationDriver):
 
     @staticmethod
     def get_connection(parsed_url, options):
-        host = options.get("host")
-        port = parsed_url.port or options.get("port")
-        dbname = parsed_url.path[1:] or options.get("dbname")
+        host = options.get("host") or parsed_url.hostname
+        port = options.get("port") or parsed_url.port
+        dbname = options.get("dbname") or parsed_url.path[1:]
         username = parsed_url.username
         password = parsed_url.password
 
