@@ -17,12 +17,11 @@
 import os
 import threading
 import time
-import uuid
 
 from concurrent import futures
 import fixtures
 import mock
-import testscenarios
+from six.moves.urllib import parse
 from testtools import matchers
 from testtools import testcase
 
@@ -39,31 +38,9 @@ def try_to_lock_job(name, coord, url, member_id):
     return lock2.acquire(blocking=False)
 
 
-class TestAPI(testscenarios.TestWithScenarios,
-              tests.TestCaseSkipNotImplemented):
+class TestAPI(tests.TestCaseSkipNotImplemented):
 
-    scenarios = [
-        ('kazoo', {'url': os.getenv("TOOZ_TEST_ZOOKEEPER_URL"),
-                   'bad_url': 'kazoo://localhost:1'}),
-        ('zake', {'url': 'zake://?timeout=5'}),
-        ('memcached', {'url': os.getenv("TOOZ_TEST_MEMCACHED_URL"),
-                       'bad_url': 'memcached://localhost:1',
-                       'timeout_capable': True}),
-        ('ipc', {'url': 'ipc://'}),
-        ('file', {'url': 'file:///tmp'}),
-        ('redis', {'url': os.getenv("TOOZ_TEST_REDIS_URL"),
-                   'bad_url': 'redis://localhost:1',
-                   'timeout_capable': True}),
-        ('postgresql', {'url': os.getenv("TOOZ_TEST_PGSQL_URL"),
-                        'bad_url': 'postgresql://localhost:1'}),
-        ('mysql', {'url': os.getenv("TOOZ_TEST_MYSQL_URL"),
-                   'bad_url': 'mysql://localhost:1'}),
-        ('zookeeper', {'url': os.getenv("TOOZ_TEST_ZOOKEEPER_URL"),
-                       'bad_url': 'zookeeper://localhost:1'}),
-        ('etcd', {'url': os.getenv("TOOZ_TEST_ETCD_URL"),
-                  'bad_url': 'etcd://localhost:1'}),
-        ('consul', {'url': os.getenv("TOOZ_TEST_CONSUL_URL")}),
-    ]
+    url = os.getenv("TOOZ_TEST_URL")
 
     def assertRaisesAny(self, exc_classes, callable_obj, *args, **kwargs):
         checkers = [matchers.MatchesException(exc_class)
@@ -74,11 +51,11 @@ class TestAPI(testscenarios.TestWithScenarios,
 
     def setUp(self):
         super(TestAPI, self).setUp()
-        self.useFixture(fixtures.NestedTempfile())
         if self.url is None:
             self.skipTest("No URL set for this driver")
-        self.group_id = self._get_random_uuid()
-        self.member_id = self._get_random_uuid()
+        self.useFixture(fixtures.NestedTempfile())
+        self.group_id = tests.get_random_uuid()
+        self.member_id = tests.get_random_uuid()
         self._coord = tooz.coordination.get_coordinator(self.url,
                                                         self.member_id)
         self._coord.start()
@@ -87,11 +64,14 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.stop()
         super(TestAPI, self).tearDown()
 
-    def test_connection_error(self):
-        if not hasattr(self, "bad_url"):
-            raise testcase.TestSkipped("No bad URL provided")
-        coord = tooz.coordination.get_coordinator(self.bad_url,
-                                                  self.member_id)
+    def test_connection_error_bad_host(self):
+        if (tooz.coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS
+           not in self._coord.CHARACTERISTICS):
+            self.skipTest("This driver is not distributed across hosts")
+        scheme = parse.urlparse(self.url).scheme
+        coord = tooz.coordination.get_coordinator(
+            "%s://localhost:1/f00" % scheme,
+            self.member_id)
         self.assertRaises(tooz.coordination.ToozConnectionError,
                           coord.start)
 
@@ -104,11 +84,11 @@ class TestAPI(testscenarios.TestWithScenarios,
     def test_create_group(self):
         self._coord.create_group(self.group_id).get()
         all_group_ids = self._coord.get_groups().get()
-        self.assertTrue(self.group_id in all_group_ids)
+        self.assertIn(self.group_id, all_group_ids)
 
     def test_get_lock_release_broken(self):
-        name = self._get_random_uuid()
-        memberid2 = self._get_random_uuid()
+        name = tests.get_random_uuid()
+        memberid2 = tests.get_random_uuid()
         coord2 = tooz.coordination.get_coordinator(self.url,
                                                    memberid2)
         coord2.start()
@@ -120,7 +100,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertTrue(lock2.acquire(blocking=False))
         self.assertFalse(lock1.release())
         # Assert lock is not accidentally broken now
-        memberid3 = self._get_random_uuid()
+        memberid3 = tests.get_random_uuid()
         coord3 = tooz.coordination.get_coordinator(self.url,
                                                    memberid3)
         coord3.start()
@@ -134,7 +114,7 @@ class TestAPI(testscenarios.TestWithScenarios,
                           create_group.get)
 
     def test_get_groups(self):
-        groups_ids = [self._get_random_uuid() for _ in range(0, 5)]
+        groups_ids = [tests.get_random_uuid() for _ in range(0, 5)]
         for group_id in groups_ids:
             self._coord.create_group(group_id).get()
         created_groups = self._coord.get_groups().get()
@@ -144,10 +124,10 @@ class TestAPI(testscenarios.TestWithScenarios,
     def test_delete_group(self):
         self._coord.create_group(self.group_id).get()
         all_group_ids = self._coord.get_groups().get()
-        self.assertTrue(self.group_id in all_group_ids)
+        self.assertIn(self.group_id, all_group_ids)
         self._coord.delete_group(self.group_id).get()
         all_group_ids = self._coord.get_groups().get()
-        self.assertFalse(self.group_id in all_group_ids)
+        self.assertNotIn(self.group_id, all_group_ids)
         join_group = self._coord.join_group(self.group_id)
         self.assertRaises(tooz.coordination.GroupNotCreated,
                           join_group.get)
@@ -170,13 +150,18 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.create_group(self.group_id).get()
         self._coord.join_group(self.group_id).get()
         member_list = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in member_list)
+        self.assertIn(self.member_id, member_list)
 
     def test_join_nonexistent_group(self):
         join_group = self._coord.join_group(self.group_id)
         self.assertRaises(tooz.coordination.GroupNotCreated,
                           join_group.get)
 
+    def test_join_group_create(self):
+        self._coord.join_group_create(self.group_id)
+        member_list = self._coord.get_members(self.group_id).get()
+        self.assertIn(self.member_id, member_list)
+
     def test_join_group_with_member_id_already_exists(self):
         self._coord.create_group(self.group_id).get()
         self._coord.join_group(self.group_id).get()
@@ -190,20 +175,20 @@ class TestAPI(testscenarios.TestWithScenarios,
     def test_leave_group(self):
         self._coord.create_group(self.group_id).get()
         all_group_ids = self._coord.get_groups().get()
-        self.assertTrue(self.group_id in all_group_ids)
+        self.assertIn(self.group_id, all_group_ids)
         self._coord.join_group(self.group_id).get()
         member_list = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in member_list)
+        self.assertIn(self.member_id, member_list)
         member_ids = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in member_ids)
+        self.assertIn(self.member_id, member_ids)
         self._coord.leave_group(self.group_id).get()
         new_member_objects = self._coord.get_members(self.group_id).get()
         new_member_list = [member.member_id for member in new_member_objects]
-        self.assertTrue(self.member_id not in new_member_list)
+        self.assertNotIn(self.member_id, new_member_list)
 
     def test_leave_nonexistent_group(self):
         all_group_ids = self._coord.get_groups().get()
-        self.assertTrue(self.group_id not in all_group_ids)
+        self.assertNotIn(self.group_id, all_group_ids)
         leave_group = self._coord.leave_group(self.group_id)
         # Drivers raise one of those depending on their capability
         self.assertRaisesAny([tooz.coordination.MemberNotJoined,
@@ -213,13 +198,13 @@ class TestAPI(testscenarios.TestWithScenarios,
     def test_leave_group_not_joined_by_member(self):
         self._coord.create_group(self.group_id).get()
         all_group_ids = self._coord.get_groups().get()
-        self.assertTrue(self.group_id in all_group_ids)
+        self.assertIn(self.group_id, all_group_ids)
         leave_group = self._coord.leave_group(self.group_id)
         self.assertRaises(tooz.coordination.MemberNotJoined,
                           leave_group.get)
 
     def test_get_lock_twice_locked_one_released_two(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         self.assertTrue(lock1.acquire())
@@ -229,8 +214,8 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertFalse(lock2.release())
 
     def test_get_members(self):
-        group_id_test2 = self._get_random_uuid()
-        member_id_test2 = self._get_random_uuid()
+        group_id_test2 = tests.get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -239,8 +224,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.join_group(group_id_test2).get()
         client2.join_group(group_id_test2).get()
         members_ids = self._coord.get_members(group_id_test2).get()
-        self.assertTrue(self.member_id in members_ids)
-        self.assertTrue(member_id_test2 in members_ids)
+        self.assertEqual({self.member_id, member_id_test2}, members_ids)
 
     def test_get_member_capabilities(self):
         self._coord.create_group(self.group_id).get()
@@ -309,7 +293,7 @@ class TestAPI(testscenarios.TestWithScenarios,
 
     def test_get_member_info_nonjoined_member(self):
         self._coord.create_group(self.group_id).get()
-        member_id = self._get_random_uuid()
+        member_id = tests.get_random_uuid()
         member_info = self._coord.get_member_info(self.group_id,
                                                   member_id)
         self.assertRaises(tooz.coordination.MemberNotJoined,
@@ -361,7 +345,7 @@ class TestAPI(testscenarios.TestWithScenarios,
             heart.wait()
 
     def test_disconnect_leave_group(self):
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -369,26 +353,35 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.join_group(self.group_id).get()
         client2.join_group(self.group_id).get()
         members_ids = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in members_ids)
-        self.assertTrue(member_id_test2 in members_ids)
+        self.assertIn(self.member_id, members_ids)
+        self.assertIn(member_id_test2, members_ids)
         client2.stop()
         members_ids = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in members_ids)
-        self.assertTrue(member_id_test2 not in members_ids)
+        self.assertIn(self.member_id, members_ids)
+        self.assertNotIn(member_id_test2, members_ids)
 
     def test_timeout(self):
-        if not getattr(self, "timeout_capable", False):
-            self.skipTest("This test only works with timeout capable drivers")
-        member_id_test2 = self._get_random_uuid()
-        client2 = tooz.coordination.get_coordinator(self.url,
-                                                    member_id_test2)
+        if (tooz.coordination.Characteristics.NON_TIMEOUT_BASED
+           in self._coord.CHARACTERISTICS):
+            self.skipTest("This driver is not based on timeout")
+        self._coord.stop()
+        if "?" in self.url:
+            sep = "&"
+        else:
+            sep = "?"
+        url = self.url + sep + "timeout=5"
+        self._coord = tooz.coordination.get_coordinator(url, self.member_id)
+        self._coord.start()
+
+        member_id_test2 = tests.get_random_uuid()
+        client2 = tooz.coordination.get_coordinator(url, member_id_test2)
         client2.start()
         self._coord.create_group(self.group_id).get()
         self._coord.join_group(self.group_id).get()
         client2.join_group(self.group_id).get()
         members_ids = self._coord.get_members(self.group_id).get()
-        self.assertTrue(self.member_id in members_ids)
-        self.assertTrue(member_id_test2 in members_ids)
+        self.assertIn(self.member_id, members_ids)
+        self.assertIn(member_id_test2, members_ids)
 
         # Watch the group, we want to be sure that when client2 is kicked out
         # we get an event.
@@ -404,8 +397,8 @@ class TestAPI(testscenarios.TestWithScenarios,
         while True:
             if self._coord.run_watchers():
                 break
-        self.assertTrue(self.member_id in members_ids)
-        self.assertTrue(member_id_test2 not in members_ids)
+        self.assertIn(self.member_id, members_ids)
+        self.assertNotIn(member_id_test2, members_ids)
         # Check that the event has been triggered
         self.assertIsInstance(self.event,
                               tooz.coordination.MemberLeftGroup)
@@ -419,7 +412,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         return 42
 
     def test_watch_group_join(self):
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -431,7 +424,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         # Join the group
         client2.join_group(self.group_id).get()
         members_ids = self._coord.get_members(self.group_id).get()
-        self.assertTrue(member_id_test2 in members_ids)
+        self.assertIn(member_id_test2, members_ids)
         while True:
             if self._coord.run_watchers():
                 break
@@ -453,7 +446,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertIsNone(self.event)
 
     def test_watch_leave_group(self):
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -504,7 +497,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.watch_join_group(self.group_id, self._set_event)
         self._coord.watch_leave_group(self.group_id, self._set_event)
 
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -540,7 +533,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.watch_join_group(self.group_id, self._set_event)
         self._coord.watch_leave_group(self.group_id, self._set_event)
 
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -557,7 +550,9 @@ class TestAPI(testscenarios.TestWithScenarios,
         # Only works for clients that have access to the groups they are part
         # of, to ensure that after we got booted out by client3 that this
         # client now no longer believes its part of the group.
-        if hasattr(self._coord, '_joined_groups'):
+        if (hasattr(self._coord, '_joined_groups')
+           and (self._coord.run_watchers
+                == tooz.coordination._RunWatchersMixin.run_watchers)):
             self.assertIn(self.group_id, self._coord._joined_groups)
             self._coord.run_watchers()
             self.assertNotIn(self.group_id, self._coord._joined_groups)
@@ -586,7 +581,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.watch_elected_as_leader(self.group_id, self._set_event)
         self._coord.run_watchers()
 
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -623,12 +618,12 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.create_group(self.group_id).get()
 
         leader = self._coord.get_leader(self.group_id).get()
-        self.assertEqual(leader, None)
+        self.assertIsNone(leader)
 
         self._coord.join_group(self.group_id).get()
 
         leader = self._coord.get_leader(self.group_id).get()
-        self.assertEqual(leader, None)
+        self.assertIsNone(leader)
 
         # Let's get elected
         self._coord.watch_elected_as_leader(self.group_id, self._set_event)
@@ -642,7 +637,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.watch_elected_as_leader(self.group_id, self._set_event)
         self._coord.run_watchers()
 
-        member_id_test2 = self._get_random_uuid()
+        member_id_test2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
         client2.start()
@@ -686,7 +681,8 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.watch_elected_as_leader(self.group_id, self._set_event)
 
         # Ensure exactly one leader election hook exists
-        self.assertEqual(1, len(self._coord._hooks_elected_leader))
+        self.assertEqual(1,
+                         len(self._coord._hooks_elected_leader[self.group_id]))
 
         # Unwatch, and ensure no leader election hooks exist
         self._coord.unwatch_elected_as_leader(self.group_id, self._set_event)
@@ -704,6 +700,19 @@ class TestAPI(testscenarios.TestWithScenarios,
                           self._coord.unwatch_join_group,
                           self.group_id, lambda x: None)
 
+    def test_unwatch_leave_group(self):
+        # Create a group and add a leave_group callback
+        self._coord.create_group(self.group_id).get()
+        self.assertEqual(0, len(self._coord._hooks_leave_group))
+        self._coord.watch_leave_group(self.group_id, self._set_event)
+
+        # Ensure exactly one leave group hook exists
+        self.assertEqual(1, len(self._coord._hooks_leave_group[self.group_id]))
+
+        # Unwatch, and ensure no leave group hooks exist
+        self._coord.unwatch_leave_group(self.group_id, self._set_event)
+        self.assertEqual(0, len(self._coord._hooks_leave_group))
+
     def test_unwatch_leave_group_callback_not_found(self):
         self._coord.create_group(self.group_id).get()
         self.assertRaises(tooz.coordination.WatchCallbackNotFound,
@@ -711,14 +720,14 @@ class TestAPI(testscenarios.TestWithScenarios,
                           self.group_id, lambda x: None)
 
     def test_get_lock(self):
-        lock = self._coord.get_lock(self._get_random_uuid())
+        lock = self._coord.get_lock(tests.get_random_uuid())
         self.assertTrue(lock.acquire())
         self.assertTrue(lock.release())
         with lock:
             pass
 
     def test_get_lock_concurrency_locking_same_lock(self):
-        lock = self._coord.get_lock(self._get_random_uuid())
+        lock = self._coord.get_lock(tests.get_random_uuid())
 
         graceful_ending = threading.Event()
 
@@ -739,13 +748,13 @@ class TestAPI(testscenarios.TestWithScenarios,
 
     def _do_test_get_lock_concurrency_locking_two_lock(self, executor,
                                                        use_same_coord):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         with lock1:
             with executor(max_workers=1) as e:
                 coord = self._coord if use_same_coord else None
                 f = e.submit(try_to_lock_job, name, coord, self.url,
-                             self._get_random_uuid())
+                             tests.get_random_uuid())
                 self.assertFalse(f.result())
 
     def test_get_lock_concurrency_locking_two_lock_process(self):
@@ -764,8 +773,8 @@ class TestAPI(testscenarios.TestWithScenarios,
         # NOTE(sileht): some database based lock can have only
         # one lock per connection, this test ensures acquiring a
         # second lock doesn't release the first one.
-        lock1 = self._coord.get_lock(self._get_random_uuid())
-        lock2 = self._coord.get_lock(self._get_random_uuid())
+        lock1 = self._coord.get_lock(tests.get_random_uuid())
+        lock2 = self._coord.get_lock(tests.get_random_uuid())
 
         graceful_ending = threading.Event()
         thread_locked = threading.Event()
@@ -791,14 +800,14 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertTrue(graceful_ending.is_set())
 
     def test_get_lock_twice_locked_twice(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         with lock1:
             self.assertFalse(lock2.acquire(blocking=False))
 
     def test_get_lock_context_fails(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         with mock.patch.object(lock2, 'acquire', return_value=False):
@@ -808,34 +817,34 @@ class TestAPI(testscenarios.TestWithScenarios,
                     lock2.__enter__)
 
     def test_get_lock_context_check_value(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock = self._coord.get_lock(name)
         with lock as returned_lock:
             self.assertEqual(lock, returned_lock)
 
     def test_lock_context_manager_acquire_no_argument(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         with lock1():
             self.assertFalse(lock2.acquire(blocking=False))
 
     def test_lock_context_manager_acquire_argument_return_value(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         blocking_value = 10.12
         lock = self._coord.get_lock(name)
         with lock(blocking_value) as returned_lock:
             self.assertEqual(lock, returned_lock)
 
     def test_lock_context_manager_acquire_argument_release_within(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         blocking_value = 10.12
         lock = self._coord.get_lock(name)
         with lock(blocking_value) as returned_lock:
             self.assertTrue(returned_lock.release())
 
     def test_lock_context_manager_acquire_argument(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         blocking_value = 10.12
         lock = self._coord.get_lock(name)
         with mock.patch.object(lock, 'acquire', wraps=True, autospec=True) as \
@@ -844,7 +853,7 @@ class TestAPI(testscenarios.TestWithScenarios,
                 mock_acquire.assert_called_once_with(blocking_value)
 
     def test_lock_context_manager_acquire_argument_timeout(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         with lock1:
@@ -855,13 +864,13 @@ class TestAPI(testscenarios.TestWithScenarios,
                 pass
 
     def test_get_lock_locked_twice(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock = self._coord.get_lock(name)
         with lock:
             self.assertFalse(lock.acquire(blocking=False))
 
     def test_get_multiple_locks_with_same_coord(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         lock2 = self._coord.get_lock(name)
         self.assertTrue(lock1.acquire())
@@ -870,19 +879,19 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertTrue(lock1.release())
 
     def test_ensure_acquire_release_return(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock1 = self._coord.get_lock(name)
         self.assertTrue(lock1.acquire())
         self.assertTrue(lock1.release())
         self.assertFalse(lock1.release())
 
     def test_get_lock_multiple_coords(self):
-        member_id2 = self._get_random_uuid()
+        member_id2 = tests.get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id2)
         client2.start()
 
-        lock_name = self._get_random_uuid()
+        lock_name = tests.get_random_uuid()
         lock = self._coord.get_lock(lock_name)
         self.assertTrue(lock.acquire())
 
@@ -899,22 +908,30 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.start()
 
     def do_test_name_property(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock = self._coord.get_lock(name)
         self.assertEqual(name, lock.name)
 
-    @staticmethod
-    def _get_random_uuid():
-        return str(uuid.uuid4()).encode('ascii')
-
     def test_acquire_twice_no_deadlock_releasing(self):
-        name = self._get_random_uuid()
+        name = tests.get_random_uuid()
         lock = self._coord.get_lock(name)
         self.assertTrue(lock.acquire(blocking=False))
         self.assertFalse(lock.acquire(blocking=False))
         self.assertTrue(lock.release())
 
 
+class ZakeTestAPI(TestAPI):
+    url = "zake://"
+
+
+class IPCTestAPI(TestAPI):
+    url = "ipc://"
+
+
+class FileTestAPI(TestAPI):
+    url = "file:///tmp"
+
+
 class TestHook(testcase.TestCase):
     def setUp(self):
         super(TestHook, self).setUp()
